@@ -39,6 +39,8 @@ struct stfl_widget_type *stfl_widget_types[] = {
 	&stfl_widget_type_list,
 	&stfl_widget_type_listitem,
 	&stfl_widget_type_textview,
+	&stfl_widget_type_textedit,
+	&stfl_widget_type_checkbox,
 	0
 };
 
@@ -336,14 +338,17 @@ struct stfl_widget *stfl_find_child_tree(struct stfl_widget *w, struct stfl_widg
 
 extern struct stfl_widget *stfl_find_first_focusable(struct stfl_widget *w)
 {
-	if (w->allow_focus)
+	if (w->allow_focus && stfl_widget_getkv_int(w, L"can_focus", 1) &&
+	    stfl_widget_getkv_int(w, L".display", 1))
 		return w;
 
 	struct stfl_widget *c = w->first_child;
 	while (c) {
-		struct stfl_widget *r = stfl_find_first_focusable(c);
-		if (r)
-			return r;
+		if (stfl_widget_getkv_int(w, L".display", 1)) {
+			struct stfl_widget *r = stfl_find_first_focusable(c);
+			if (r)
+				return r;
+		}
 		c = c->next_sibling;
 	}
 
@@ -556,7 +561,7 @@ void stfl_form_run(struct stfl_form *f, int timeout)
 			goto unshift_next_event;
 		}
 
-		if (w->type->f_process && w->type->f_process(w, fw, f, wch, rc == KEY_CODE_YES))
+		if (w->type->f_process && stfl_widget_getkv_int(w, L"process", 1) && w->type->f_process(w, fw, f, wch, rc == KEY_CODE_YES))
 			goto unshift_next_event;
 
 		if (stfl_widget_getkv_int(w, L"modal", 0))
@@ -587,7 +592,7 @@ void stfl_form_run(struct stfl_form *f, int timeout)
 
 			if (!fw && old_fw)
 				fw = f->root;
-		} while (fw && !fw->allow_focus);
+		} while (fw && !(fw->allow_focus && stfl_widget_getkv_int(fw, L"can_focus", 1)));
 
 		if (old_fw != fw)
 		{
@@ -602,6 +607,50 @@ void stfl_form_run(struct stfl_form *f, int timeout)
 
 		goto unshift_next_event;
 	}
+	else if (rc == KEY_CODE_YES && wch == KEY_BTAB)
+	{
+		struct stfl_widget *old_fw = stfl_widget_by_id(f->root, f->current_focus_id);
+		struct stfl_widget *tmp_fw = f->root;
+		struct stfl_widget *fw = 0;
+
+focus_wrap_around:
+		while (tmp_fw && tmp_fw != old_fw)
+		{
+			if (tmp_fw->allow_focus && stfl_widget_getkv_int(tmp_fw, L"can_focus", 1))
+				fw = tmp_fw;
+
+			if (tmp_fw->first_child)
+				tmp_fw = tmp_fw->first_child;
+			else
+			if (tmp_fw->next_sibling)
+				tmp_fw = tmp_fw->next_sibling;
+			else
+			{
+				while (tmp_fw->parent && !tmp_fw->parent->next_sibling)
+					tmp_fw = tmp_fw->parent;
+				tmp_fw = tmp_fw->parent ? tmp_fw->parent->next_sibling : 0;
+			}
+		}
+
+		if (!fw && old_fw)
+		{
+			old_fw = f->root->last_child;
+			goto focus_wrap_around;
+		}
+
+		if (fw && old_fw != fw)
+		{
+			if (old_fw && old_fw->type->f_leave)
+				old_fw->type->f_leave(old_fw, f);
+
+			if (fw && fw->type->f_enter)
+				fw->type->f_enter(fw, f);
+
+			f->current_focus_id = fw ? fw->id : 0;
+		}
+
+		goto unshift_next_event;
+	}
 
 generate_event:
 	stfl_form_event(f, stfl_keyname(wch, rc == KEY_CODE_YES));
@@ -626,6 +675,12 @@ void stfl_form_reset()
 	}
 }
 
+void stfl_form_redraw()
+{
+	if (curses_active)
+		clearok(curscr, 1);
+}
+
 void stfl_form_free(struct stfl_form *f)
 {
 	pthread_mutex_lock(&f->mtx);
