@@ -97,6 +97,16 @@ static struct spl_node *handler_stfl_run(struct spl_task *task, void *data)
 	return ret;
 }
 
+/**
+ * Instruct STFL to completely redraw screen on next run
+ */
+// builtin stfl_redraw()
+static struct spl_node *handler_stfl_redraw(struct spl_task *task, void *data)
+{
+	stfl_redraw();
+	return 0;
+}
+
 /**
  * Return to standard text mode
  */
@@ -184,6 +194,20 @@ static struct spl_node *handler_stfl_dump(struct spl_task *task, void *data)
 	return n;
 }
 
+/**
+ * Dump the text under a widget
+ */
+// builtin stfl_text(form, name)
+static struct spl_node *handler_stfl_text(struct spl_task *task, void *data)
+{
+	struct stfl_form *f = clib_get_stfl_form(task);
+	char *name = spl_clib_get_string(task);
+	const char *text = stfl_ipool_fromwc(ipool, stfl_text(f, stfl_ipool_towc(ipool, name)));
+	struct spl_node *n = spl_new_nullable_ascii(text);
+	stfl_ipool_flush(ipool);
+	return n;
+}
+
 /**
  * Import STFL code to an existing form
  */
@@ -199,20 +223,6 @@ static struct spl_node *handler_stfl_modify(struct spl_task *task, void *data)
 	return 0;
 }
 
-/**
- * Lookup widgets in the form using a path and optionally assign a new name
- */
-// builtin stfl_lookup(form, path, newname)
-static struct spl_node *handler_stfl_lookup(struct spl_task *task, void *data)
-{
-	struct stfl_form *f = clib_get_stfl_form(task);
-	char *path = spl_clib_get_string(task);
-	char *newname = spl_clib_get_string(task);
-	struct spl_node *ret = spl_new_nullable_ascii(stfl_ipool_fromwc(ipool, stfl_lookup(f, stfl_ipool_towc(ipool, path), stfl_ipool_towc(ipool, newname))));
-	stfl_ipool_flush(ipool);
-	return ret;
-}
-
 /**
  * Return error message of last stfl call or undef.
  */
@@ -255,6 +265,7 @@ void SPL_ABI(spl_mod_stfl_init)(struct spl_vm *vm, struct spl_module *mod, int r
 	spl_clib_reg(vm, "stfl_create", handler_stfl_create, 0);
 
 	spl_clib_reg(vm, "stfl_run", handler_stfl_run, 0);
+	spl_clib_reg(vm, "stfl_redraw", handler_stfl_redraw, 0);
 	spl_clib_reg(vm, "stfl_reset", handler_stfl_reset, 0);
 
 	spl_clib_reg(vm, "stfl_get", handler_stfl_get, 0);
@@ -267,8 +278,8 @@ void SPL_ABI(spl_mod_stfl_init)(struct spl_vm *vm, struct spl_module *mod, int r
 	spl_clib_reg(vm, "stfl_quote", handler_stfl_quote, 0);
 
 	spl_clib_reg(vm, "stfl_dump", handler_stfl_dump, 0);
+	spl_clib_reg(vm, "stfl_text", handler_stfl_text, 0);
 	spl_clib_reg(vm, "stfl_modify", handler_stfl_modify, 0);
-	spl_clib_reg(vm, "stfl_lookup", handler_stfl_lookup, 0);
 
 	spl_clib_reg(vm, "stfl_error", handler_stfl_error, 0);
 	spl_clib_reg(vm, "stfl_error_action", handler_stfl_error_action, 0);
