@@ -97,6 +97,8 @@ extern struct stfl_widget_type stfl_widget_type_tablebr;
 extern struct stfl_widget_type stfl_widget_type_list;
 extern struct stfl_widget_type stfl_widget_type_listitem;
 extern struct stfl_widget_type stfl_widget_type_textview;
+extern struct stfl_widget_type stfl_widget_type_textedit;
+extern struct stfl_widget_type stfl_widget_type_checkbox;
 
 extern struct stfl_widget *stfl_widget_new(const wchar_t *type);
 extern void stfl_widget_free(struct stfl_widget *w);
@@ -132,6 +134,7 @@ extern void stfl_form_event(struct stfl_form *f, wchar_t *event);
 extern void stfl_form_run(struct stfl_form *f, int timeout);
 extern void stfl_form_reset();
 extern void stfl_form_free(struct stfl_form *f);
+extern void stfl_form_redraw();
 
 extern void stfl_check_setfocus(struct stfl_form *f, struct stfl_widget *w);
 
@@ -140,6 +143,7 @@ extern struct stfl_widget *stfl_parser_file(const char *filename);
 
 extern wchar_t *stfl_quote_backend(const wchar_t *text);
 extern wchar_t *stfl_widget_dump(struct stfl_widget *w, const wchar_t *prefix, int focus_id);
+extern wchar_t *stfl_widget_text(struct stfl_widget *w);
 
 extern void stfl_style(WINDOW *win, const wchar_t *style);
 extern void stfl_widget_style(struct stfl_widget *w, struct stfl_form *f, WINDOW *win);
